/**
 * This factory is responsible for creation of metadata for axway-requester.
 * axway-requester use it for communicating properly with third party services.
 *
 * @returns {object} object that contains requester metadata and function that adds more
 * 		metadata to it.
 */
module.exports = () => {
	const methodsMetadata = {};

	return { methodsMetadata, addMethodMetadata };

	function addMethodMetadata ({
		customOperationID,
		mpath,
		httpMethod,
		securityDefinitions,
		method,
		methodSpec
	}) {
		methodsMetadata[customOperationID] = {
			path: mpath,
			httpMethod,
			security: method.security,
			securityDefinitions,
			parameters: constructParameters(method, methodSpec)
		};
	}

	function constructParameters(method, methodSpec) {
		const orderedParameters = {
			query: [],
			header: [],
			path: [],
			body: [],
			formData: [],
			// This holds the default values for some of the advanced HTTP parameters.
			// Usually we can hardcode the default values but in this case the default values
			// are dynamic so we have to pass them down.
			defaultValues: []
		};
		const parameters = method.parameters || [];
		for (const param of parameters) {
			const place = param.in;
			orderedParameters[place].push(param);
		}

		// Add additional parameter mapping for http parameters in the spec but not in swagger
		for (const id of [ '_content-type', '_accept' ]) {
			const param = methodSpec.parameter.properties[id];
			if (param) {
				// Add it as parameter
				orderedParameters.header.push({ name: id });
				// Store its default value to use it in case it is not provided
				orderedParameters.defaultValues.push({
					in: 'header',
					name: id,
					defaultValue: param.default
				});
			}
		}
		return orderedParameters;
	}
};
